#  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License").
#    You may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import logging
import random
import string
from typing import Any, Callable, Dict, List, Optional, Tuple, Union

from boto3 import Session

from seedfarmer.commands import _cfn_seedkit as cfn_seedkit
from seedfarmer.services import _cfn as cfn
from seedfarmer.services import _s3 as s3

_logger: logging.Logger = logging.getLogger(__name__)


def seedkit_deployed(
    seedkit_name: str, session: Optional[Union[Callable[[], Session], Session]] = None
) -> Tuple[bool, str, Dict[str, str]]:
    """Checks for existence of the Seedkit CloudFormation Stack

    If the Stack exists, then the Stack Outputs are also returned to eliminate need for another roundtrip call to
    CloudFormation.

    Parameters
    ----------
    seedkit_name : str
        Named of the seedkit to check.
    session: Optional[Union[Callable[[], Session], Session]], optional
        Optional Session or function returning a Session to use for all boto3 operations, by default None

    Returns
    -------
    Tuple[bool, str, Dict[str, str]]
        Returns a Tuple with a bool indicating existence of the Stack, the Stack name, and a dict with the
        Stack Outputs
    """
    stack_name: str = cfn.get_stack_name(seedkit_name=seedkit_name)
    stack_exists, stack_outputs = cfn.does_stack_exist(stack_name=stack_name, session=session)
    return stack_exists, stack_name, stack_outputs


def deploy_seedkit(
    seedkit_name: str,
    managed_policy_arns: Optional[List[str]] = None,
    deploy_codeartifact: bool = False,
    session: Optional[Session] = None,
    vpc_id: Optional[str] = None,
    subnet_ids: Optional[List[str]] = None,
    security_group_ids: Optional[List[str]] = None,
    permissions_boundary_arn: Optional[str] = None,
    synthesize: bool = False,
    **kwargs: Dict[str, Any],
) -> None:
    """Deploys the seedkit resources into the environment.

    Resources deployed include: S3 Bucket, CodeArtifact Domain, CodeArtifact Repository, CodeBuild Project,
    IAM Role, IAM Managed Policy, and KMS Key. All resource names will include the seedkit_name and IAM Role and Policy
    grant least privilege access to only the resources associated with this Seedkit. Seedkits are deployed to an
    AWS Region, names on global resources (S3, IAM) include a region identifier to avoid conflicts and ensure the same
    Seedkit name can be deployed to multiple regions.

    Parameters
    ----------
    seedkit_name : str
        Name of the seedkit to deploy. All resources will include this in their naming conventions
    managed_policy_arns : Optional[List[str]]
        List of Managed Policy to ARNs to attach to the default IAM Role created and used
        by the CodeBuild Project
    deploy_codeartifact : bool
        Trigger optional deployment of CodeArtifact Domain and Repository for use by the Seedkit and
        its libraries
    session: Optional[Union[Callable[[], Session], Session]], optional
        Optional Session or function returning a Session to use for all boto3 operations, by default None
    vpc_id: Optional[str]
        If deploying codebuild in a VPC, the VPC-ID to use
        (must have vpc-id, subnets, and security_group_ids)
    subnet_ids:  Optional[List[str]]
        If deploying codebuild in a VPC, a list of Subnets to use
        (must have vpc-id, subnets, and security_group_ids)
    security_group_ids: Optional[List[str]]
        If deploying codebuild in a VPC, a list of Security Group IDs to use
        (must have vpc-id, subnets, and security_group_ids)
    permissions_boundary_arn: Optional[str]
        If using a permissions boundary, the arn of that policy to be provided
    synthesize: bool
        Synthesize seedkit template only. Do not deploy. False by default.
    """
    deploy_id: Optional[str] = None
    stack_exists, stack_name, stack_outputs = seedkit_deployed(seedkit_name=seedkit_name, session=session)
    _logger.info("Deploying Seedkit %s with Stack Name %s", seedkit_name, stack_name)
    _logger.debug("Managed Policy Arns: %s", managed_policy_arns)
    _logger.debug("VPC-ID: %s", vpc_id)
    _logger.debug("Subnets: %s", subnet_ids)
    _logger.debug("Security Groups %s", security_group_ids)
    if stack_exists:
        deploy_id = stack_outputs.get("DeployId")
        _logger.info("Seedkit found with DeployId: %s", deploy_id)

    deploy_id = deploy_id if deploy_id else "".join(random.choice(string.ascii_lowercase) for i in range(6))
    template_filename: Optional[str] = cfn_seedkit.synth(
        deploy_id=deploy_id,
        synthesize=synthesize,
        **kwargs,
    )

    # Create parameters dictionary for CloudFormation
    parameters: Dict[str, str] = {
        "SeedkitName": seedkit_name,
        "DeployId": deploy_id,
        "RolePrefix": str(kwargs.get("role_prefix", "/")),
        "PolicyPrefix": str(kwargs.get("policy_prefix", "/")),
        "DeployCodeArtifact": str(deploy_codeartifact).lower(),
    }

    # Only add ManagedPolicyArns if it's not None
    if managed_policy_arns:
        parameters["ManagedPolicyArns"] = ",".join(managed_policy_arns)

    # Only add PermissionsBoundaryArn if it's not None
    if permissions_boundary_arn:
        parameters["PermissionsBoundaryArn"] = permissions_boundary_arn

    # Only add VpcId if it's not None
    if vpc_id:
        parameters["VpcId"] = vpc_id

    # Only add SecurityGroupIds if it's not None
    if security_group_ids:
        parameters["SecurityGroupIds"] = ",".join(security_group_ids)

    # Only add SubnetIds if it's not None
    if subnet_ids:
        parameters["SubnetIds"] = ",".join(subnet_ids)

    if not synthesize:
        assert template_filename is not None, "Template filename is required"
        cfn.deploy_template(
            stack_name=stack_name,
            filename=template_filename,
            seedkit_tag=f"codeseeder-{seedkit_name}",  # (LEGACY)
            session=session,
            parameters=parameters,
        )
        _logger.info("Seedkit Deployed")


def destroy_seedkit(seedkit_name: str, session: Optional[Union[Callable[[], Session], Session]] = None) -> None:
    """Destroys the resources associated with the seedkit.

    Parameters
    ----------
    seedkit_name : str
        Name of the seedkit to destroy
    session: Optional[Union[Callable[[], Session], Session]], optional
        Optional Session or function returning a Session to use for all boto3 operations, by default None
    """
    stack_exists, stack_name, stack_outputs = seedkit_deployed(seedkit_name=seedkit_name, session=session)
    _logger.info("Destroying Seedkit %s with Stack Name %s", seedkit_name, stack_name)
    if stack_exists:
        seedkit_bucket = stack_outputs.get("Bucket")
        if seedkit_bucket:
            s3.delete_bucket(bucket=seedkit_bucket, session=session)
        cfn.destroy_stack(stack_name=stack_name, session=session)
        _logger.info("Seedkit Destroyed")
    else:
        _logger.warning("Seedkit/Stack does not exist")



#  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License").
#    You may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import json
import logging
import time
from typing import Any, Callable, Dict, List, Optional, Union, cast

from boto3 import Session

import seedfarmer.errors
from seedfarmer.services._service_utils import boto3_client

_logger: logging.Logger = logging.getLogger(__name__)


def put_parameter(
    name: str, obj: Dict[str, Any], session: Optional[Union[Callable[[], Session], Session]] = None
) -> None:
    client = boto3_client(service_name="ssm", session=session)
    retries = 3
    while retries > 0:
        try:
            client.put_parameter(
                Name=name,
                Value=str(json.dumps(obj=obj, sort_keys=True)),
                Overwrite=True,
                Tier="Intelligent-Tiering",
                Type="String",
            )
            break
        except client.exceptions.TooManyUpdates as err:
            if retries < 3:
                _logger.warning("An error occurred (TooManyUpdates) when calling the PutParameter operation. Retrying")
                retries -= 1
                time.sleep(2 ** (4 - retries))

            if retries == 0:
                raise seedfarmer.errors.SeedFarmerException(err)  # type: ignore[arg-type]


def get_parameter(name: str, session: Optional[Union[Callable[[], Session], Session]] = None) -> Dict[str, Any]:
    client = boto3_client(service_name="ssm", session=session)
    json_str: str = client.get_parameter(Name=name)["Parameter"]["Value"]
    try:
        return cast(Dict[str, Any], json.loads(json_str))
    except json.decoder.JSONDecodeError:
        _logger.warn("Parameter %s cannot be parsed, returning it as-is - %s ", name, json_str)
        return cast(Dict[str, Any], json_str)


def get_parameter_if_exists(
    name: str, session: Optional[Union[Callable[[], Session], Session]] = None
) -> Optional[Dict[str, Any]]:
    client = boto3_client(service_name="ssm", session=session)
    try:
        json_str: str = client.get_parameter(Name=name)["Parameter"]["Value"]
    except client.exceptions.ParameterNotFound:
        return None
    return cast(Dict[str, Any], json.loads(json_str))


def does_parameter_exist(name: str, session: Optional[Union[Callable[[], Session], Session]] = None) -> bool:
    client = boto3_client(service_name="ssm", session=session)
    try:
        client.get_parameter(Name=name)
        return True
    except client.exceptions.ParameterNotFound:
        return False


def list_parameters(prefix: str, session: Optional[Union[Callable[[], Session], Session]] = None) -> List[str]:
    client = boto3_client(service_name="ssm", session=session)
    paginator = client.get_paginator("describe_parameters")
    response_iterator = paginator.paginate(
        ParameterFilters=[
            {
                "Key": "Type",
                "Option": "Equals",
                "Values": [
                    "String",
                ],
            },
            {"Key": "Name", "Option": "BeginsWith", "Values": [prefix]},
        ],
    )
    ret: List[str] = []
    for page in response_iterator:
        for par in page["Parameters"]:
            ret.append(par["Name"])
    return ret


def list_parameters_with_filter(
    prefix: str, contains_string: str, session: Optional[Union[Callable[[], Session], Session]] = None
) -> List[str]:
    client = boto3_client(service_name="ssm", session=session)
    paginator = client.get_paginator("describe_parameters")

    response_iterator = paginator.paginate(
        ParameterFilters=[
            {
                "Key": "Type",
                "Option": "Equals",
                "Values": [
                    "String",
                ],
            },
            {"Key": "Name", "Option": "Contains", "Values": [contains_string]},
        ],
    )
    ret: List[str] = []
    for page in response_iterator:
        for par in page["Parameters"]:
            if str(par["Name"]).startswith(f"{prefix}"):
                ret.append(par["Name"])
    return ret


def get_all_parameter_data_by_path(
    prefix: str, session: Optional[Union[Callable[[], Session], Session]] = None
) -> Dict[str, Union[str, Dict[str, Any]]]:
    client = boto3_client(service_name="ssm", session=session)
    paginator = client.get_paginator("get_parameters_by_path")
    response_iterator = paginator.paginate(
        Path=prefix,
        Recursive=True,
        ParameterFilters=[
            {
                "Key": "Type",
                "Option": "Equals",
                "Values": [
                    "String",
                ],
            },
        ],
    )
    ret: Dict[str, Union[str, Dict[str, Any]]] = {}
    for page in response_iterator:
        for par in page["Parameters"]:
            try:
                ret[par["Name"]] = json.loads(par["Value"])
            except json.decoder.JSONDecodeError:
                _logger.warn("Parameter %s cannot be parsed, returning it as-is", par["Name"])
                ret[par["Name"]] = par["Value"]
    return ret


def delete_parameters(parameters: List[str], session: Optional[Union[Callable[[], Session], Session]] = None) -> None:
    if parameters:
        if len(parameters) < 10:
            _logger.debug("deleting parameters: %s", parameters)
            client = boto3_client(service_name="ssm", session=session)
            client.delete_parameters(Names=parameters)
        else:
            delete_parameters(parameters[0:9], session=session)
            delete_parameters(parameters[9:], session=session)


def describe_parameter(name: str, session: Optional[Union[Callable[[], Session], Session]] = None) -> Optional[Any]:
    client = boto3_client(service_name="ssm", session=session)
    return client.describe_parameters(
        ParameterFilters=[
            {
                "Key": "Type",
                "Option": "Equals",
                "Values": [
                    "String",
                ],
            },
            {"Key": "Name", "Option": "Equals", "Values": [name]},
        ],
    )



#  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License").
#    You may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.


import logging

import click

from seedfarmer import DEBUG_LOGGING_FORMAT, enable_debug
from seedfarmer.commands import get_default_project_policy

_logger: logging.Logger = logging.getLogger(__name__)


@click.group(
    name="projectpolicy",
    help="""Fetch info about the project policy.
      This will output the default provided project polocy that can be customized.""",
)
def projectpolicy() -> None:
    """Get info about the Project Policy"""
    pass


@projectpolicy.command(
    name="synth",
    help="Synth a Project Policy from seed-farmer.",
)
@click.option(
    "--policy-prefix",
    default="/",
    help="An IAM path prefix to use with the policy.",
    required=False,
)
@click.option("--debug/--no-debug", default=False, help="Enable detail logging", show_default=True)
def policy_synth(
    policy_prefix: str,
    debug: bool,
) -> None:
    if debug:
        enable_debug(format=DEBUG_LOGGING_FORMAT)

    get_default_project_policy(policy_prefix=policy_prefix)



#  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License").
#    You may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import logging
import sys
from typing import Optional

import click

from seedfarmer import config
from seedfarmer.mgmt import bundle_support
from seedfarmer.output_utils import print_bolded
from seedfarmer.services.session_manager import SessionManager, SessionManagerLocalImpl

_logger: logging.Logger = logging.getLogger(__name__)


def _load_project() -> str:
    try:
        return config.PROJECT
    except FileNotFoundError:
        print_bolded("Unable to determine project to bootstrap, seedfarmer.yaml is required")
        raise click.ClickException("Failed to determine project identifier")


@click.group(name="bundle", help="Manage the bundle in a module deployment execution")
def bundle() -> None:
    """Bundle Management group"""
    pass


@bundle.command(
    name="store",
    help="""Store the bundle used to deploy a module.
     This command is meant to be run by SeedFarmer ONLY!!!
    """,
)
@click.option(
    "--deployment",
    "-d",
    type=str,
    help="The Deployment Name",
    required=True,
)
@click.option(
    "--group",
    "-g",
    type=str,
    help="The Group Name",
    required=True,
)
@click.option(
    "--module",
    "-m",
    type=str,
    help="The Module Name",
    required=True,
)
@click.option(
    "--bucket",
    "-b",
    help="The name of the SeedFarmer bucket the bundle to be stored in",
    required=True,
)
@click.option(
    "--origin",
    "-o",
    help="Full path of the bundle object in SeedKit bucket",
    required=True,
)
@click.option(
    "--region",
    default=None,
    help="The AWS region used to create a session",
    required=False,
)
def store_bundle(
    deployment: str, group: str, module: str, bucket: str, origin: str, region: Optional[str] = None
) -> None:
    _load_project()
    print(f"{deployment} - {group} - {module} -{bucket} - {origin}")
    SessionManager.bind(SessionManagerLocalImpl())
    session = (
        SessionManager()
        .get_or_create(region_name=region)
        .get_deployment_session(account_id="000000000000", region_name=str(region))
    )

    bundle_support.copy_bundle_to_sf(
        deployment=deployment, group=group, module=module, bucket=bucket, bundle_src_path=origin, session=session
    )


@bundle.command(
    name="fetch",
    help="""Fetch the full path where the bundle is stored in SF.
     This command is meant to be run by SeedFarmer ONLY!!!
    """,
)
@click.option(
    "--deployment",
    "-d",
    type=str,
    help="The Deployment Name",
    required=True,
)
@click.option(
    "--group",
    "-g",
    type=str,
    help="The Group Name",
    required=True,
)
@click.option(
    "--module",
    "-m",
    type=str,
    help="The Module Name",
    required=True,
)
@click.option(
    "--bucket",
    "-b",
    type=str,
    help="The Name of the bucket where the bundle is stored in SeedFarmer",
    required=True,
)
def fetch_bundle(deployment: str, group: str, module: str, bucket: str, region: Optional[str] = None) -> None:
    _load_project()
    p = bundle_support.get_bundle_sf_path(deployment=deployment, group=group, module=module, bucket=bucket)
    sys.stdout.write(p) if p else None


@bundle.command(
    name="delete",
    help="""Delete the bundle stored in SF.
     This command is meant to be run by SeedFarmer ONLY!!!
    """,
)
@click.option(
    "--deployment",
    "-d",
    type=str,
    help="The Deployment Name",
    required=True,
)
@click.option(
    "--group",
    "-g",
    type=str,
    help="The Group Name",
    required=True,
)
@click.option(
    "--module",
    "-m",
    type=str,
    help="The Module Name",
    required=True,
)
@click.option(
    "--bucket",
    "-b",
    type=str,
    help="The Name of the bucket where the bundle is stored in SeedFarmer",
    required=True,
)
@click.option(
    "--region",
    default=None,
    help="The AWS region used to create a session",
    required=False,
)
def delete_bundle(deployment: str, group: str, module: str, bucket: str, region: Optional[str] = None) -> None:
    _load_project()
    SessionManager.bind(SessionManagerLocalImpl())
    session = (
        SessionManager().get_or_create().get_deployment_session(account_id="000000000000", region_name=str(region))
    )

    bundle_support.delete_bundle_from_sf(
        deployment=deployment, group=group, module=module, bucket=bucket, session=session
    )



#  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License").
#    You may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.


import hashlib
import os
from pathlib import Path
from typing import Any, Dict, List, Optional

from gitignore_parser import parse_gitignore

from seedfarmer.models.manifests._module_manifest import DataFile


def _evaluate_gitignore(project_path: str, module_path: str) -> Dict[str, Any]:
    ignore_paths: List[str] = []
    ignore_maps: Dict[str, Any] = {}

    def _get_paths(working_dir: str) -> None:
        gitignore_path = os.path.join(working_dir, ".gitignore")
        ignore_paths.append(gitignore_path) if os.path.exists(gitignore_path) else None
        if os.path.realpath(working_dir) != os.path.realpath(project_path):
            _get_paths(str(Path(os.path.join(working_dir, os.pardir)).resolve()))

    # If the .gitignore path exists, parse_gitignore returns a function that is callable
    _get_paths(os.path.join(project_path, module_path))
    for ignore_path in ignore_paths:
        if os.path.exists(ignore_path):
            ignore_maps[ignore_path] = parse_gitignore(ignore_path)

    return ignore_maps


def _evaluate_file(filename: str, ignore_maps: Dict[str, Any]) -> bool:
    # The map of functions representing .gitignore is called with
    # the filename as a parameter, returning a boolean.
    # If the filename is present .gitignore return True
    # else return False
    if ignore_maps is None:
        return False

    for ignore_key in ignore_maps.keys():
        if ignore_maps.get(ignore_key) is not None:
            in_ignore = (ignore_maps.get(ignore_key))(filename)  # type: ignore
            if in_ignore:
                return True
    return False


def _generate_file_hash(filepath: str) -> str:
    hash = hashlib.md5()
    blocksize = 64 * 1024

    if not os.path.exists(filepath):
        return hash.hexdigest()

    with open(filepath, "rb") as fp:
        while True:
            data = fp.read(blocksize)
            if not data:
                break
            hash.update(data)
    digest = hash.hexdigest()
    return digest


def _consolidate_hash(hashlist: List[str]) -> str:
    hash = hashlib.md5()
    for hashvalue in sorted(hashlist):
        hash.update(hashvalue.encode("utf-8"))
    return hash.hexdigest()


def get_module_md5(
    project_path: str,
    module_path: str,
    data_files: Optional[List[DataFile]] = None,
    excluded_files: Optional[List[str]] = [],
) -> str:
    """
    This will generate an MD5 of the module source code, respecting .gitingore starting at
    the module level

    Parameters
    ----------
    project_path : str
       The OPS_ROOT full path (full path of the project)
    module_path : str
        The relative path of the module code (relative to OPS_ROOT)
    data_files: Optional[List[DataFile]]
        List of DataFile objects to be packaged in the bundle
    excluded_files : List[str], optional
        A list of additional files not in .gitignore that will be exclude from the bundle md5
            NOTE: this list of files is ONLY at the module level, not subdirecties of
            the module...use .gitignore for that

    Returns
    -------
    str
        the md5 of the module code
    """
    ignore_maps = _evaluate_gitignore(project_path=project_path, module_path=module_path)

    excluded_files = [] if excluded_files is None else excluded_files

    all_files = []

    def scandir(dirname: str) -> List[str]:
        files = [
            f.path
            for f in os.scandir(dirname)
            if f.is_file() and os.path.split(f)[1] not in excluded_files and not _evaluate_file(f.path, ignore_maps)
        ]
        all_files.extend(files)
        subfolders = [f.path for f in os.scandir(dirname) if f.is_dir()]
        for dirname in list(subfolders):
            # ignore all hidden directories and any dir already in .gitignore
            (
                subfolders.extend(scandir(dirname))
                if not os.path.split(dirname)[1].startswith(".") and not _evaluate_file(dirname, ignore_maps)
                else None
            )
        return subfolders

    _ = scandir(os.path.join(project_path, module_path))

    # Add in the extra files
    if data_files is not None:
        for data_file in data_files:
            (
                all_files.append(os.path.join(data_file.get_local_file_path(), data_file.file_path))  # type: ignore
                if os.path.isfile(os.path.join(data_file.get_local_file_path(), data_file.file_path))  # type: ignore
                else None
            )

    hashvalues: List[str] = []
    for viable_file in all_files:
        hashvalues.append(_generate_file_hash(viable_file))
    return _consolidate_hash(hashvalues)



#!/usr/bin/env python

import argparse
import base64
import json
import logging
import os
import subprocess
from typing import Dict, Optional, cast
from urllib.parse import urlparse

import boto3
from botocore.exceptions import ClientError

logger = logging.getLogger()
logger.setLevel(logging.INFO)


def get_secret(secret_name: str) -> Dict[str, Dict[str, str]]:
    region_name = os.environ.get("AWS_DEFAULT_REGION")
    try:
        session = boto3.session.Session()
        client = session.client(service_name="secretsmanager", region_name=region_name)
        get_secret_value_response = client.get_secret_value(SecretId=secret_name)
    except ClientError as ce:
        logger.info("Secret with SecretId '%s' could not be retrieved from SecretsManager", secret_name)
        print(f"Secret with SecretId {secret_name} could not be retrieved from SecretsManager - {ce}")
        exit(1)
    except FileNotFoundError:
        logger.info("Make sure AWW credentials are set")
        print("Make sure credentials  AWW credentials are set")
        exit(1)
    else:
        return cast(Dict[str, Dict[str, str]], json.loads(get_secret_value_response.get("SecretString", "{}")))


def get_auth(username: Optional[str] = None, password: Optional[str] = None, ssl_token: Optional[str] = None) -> str:
    if ssl_token:
        logger.info("ssl_token found and being used in url")
        return ssl_token

    if (username and not password) or (not username and password):
        logger.error("For NPM mirror auth support, both username and password must be provided")
        raise RuntimeError("For NPM mirror auth support, both username and password must be provided")
    else:
        logger.info("Both username and password found, encoding for use in url")
        return base64.b64encode((f"{username}:{password}").encode("ascii")).decode("ascii")


def main(url: str) -> None:
    secret_name = os.environ.get("SEEDFARMER_NPM_MIRROR_SECRET", "NO_SECRET")
    # Backwards Compatibility
    if secret_name == "NO_SECRET":
        secret_name = os.environ.get("AWS_CODESEEDER_NPM_MIRROR_SECRET", "NO_SECRET")
    elif secret_name == "NO_SECRET":
        secret_name = os.environ.get("AWS_CODESEEDER_MIRROR_SECRET", "NO_SECRET")

    if secret_name != "NO_SECRET":
        secret_name_key = secret_name.split("::")[0] if "::" in secret_name else secret_name
        key = secret_name.split("::")[1] if "::" in secret_name else "npm"
        creds = get_secret(secret_name=secret_name_key)
        if key in creds.keys():
            username = creds[key].get("username") if creds[key].get("username") else None
            password = creds[key].get("password") if creds[key].get("password") else None
            ssl_token = creds[key].get("ssl_token") if creds[key].get("ssl_token") else None
            try:
                auth = get_auth(username, password, ssl_token)
            except RuntimeError:
                logger.error(f"The auth token could not be generated - check the secret {secret_name}")
                exit(1)
            registry_url = urlparse(url).netloc
            npm_key = f"//{registry_url}/:_auth"
            process = subprocess.Popen(
                ["npm", "config", "set", npm_key, auth],
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            )
            process.communicate()
            print(f"Setting npm config WITH auth:  {url}")
    else:
        logger.info("'NPM_MIRROR_SECRET' is not set")
        print("'NPM_MIRROR_SECRET' is not set")
        print(f"Setting npm config WITHOUT auth:  {url}")
    subprocess.call(["npm", "config", "set", "registry", url])


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="URL to evaluate")
    parser.add_argument("url", type=str, help="The url to set in npm'")
    args = parser.parse_args()
    main(args.url)



#  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License").
#    You may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from typing import Optional, Union

from pydantic import model_validator

from seedfarmer.errors.seedfarmer_errors import InvalidManifestError
from seedfarmer.models._base import CamelModel


class ProjectSpec(CamelModel):
    """
    ProjectSpec
    This represents the project configuration specification. This class is
    typically populated from the project's top-level seedfarmer.yaml file.
    """

    project: str
    description: Optional[str] = None
    project_policy_path: Optional[str] = None
    seedfarmer_version: Optional[Union[int, str]] = None
    manifest_validation_fail_on_unknown_fields: bool = False

    @model_validator(mode="after")
    def check_for_extra_fields(self) -> "ProjectSpec":
        if self.manifest_validation_fail_on_unknown_fields and self.model_extra:
            raise InvalidManifestError(f"The following keys are not allowed: {self.model_extra}")

        return self



#  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License").
#    You may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import dataclasses
import enum


class EnvVarType(str, enum.Enum):
    PLAINTEXT = "PLAINTEXT"
    PARAMETER_STORE = "PARAMETER_STORE"
    SECRETS_MANAGER = "SECRETS_MANAGER"


@dataclasses.dataclass()
class EnvVar:
    """EnvVar dataclass

    This class is used to define environment variables made available inside of CodeBuild. Use of this
    class enables declaration of all environment variable types that CodeBuild supports.

    Parameters
    ----------
    value : string
        The value for the environment variable. The effect of this value varies depending on the type
        of environment variable created. See the AWS official documentation for usage information:
        https://docs.aws.amazon.com/codebuild/latest/userguide/build-spec-ref.html#build-spec.env.variables
        https://docs.aws.amazon.com/codebuild/latest/userguide/build-spec-ref.html#build-spec.env.parameter-store
        https://docs.aws.amazon.com/codebuild/latest/userguide/build-spec-ref.html#build-spec.env.secrets-manager
    type : EnvVarType
        The type of environment variable: PLAINTEXT, PARAMETER_STORE, or SECRETS_MANAGER. See the AWS
        official documentation for usage, by default PLAINTEXT
    """

    value: str
    type: EnvVarType = EnvVarType.PLAINTEXT
